// Copyright (C) - lucien.povy@free.fr.  Carantec 2017.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
// ====================================================================
//We compute matrice of frequencies, magnitudes, phases of SIMO systems, continuous with or without delay AND sampled systems.
// ==================================================================== 
function [fd,dd,pd] = dpf_cd(gc,gd,fmin,fmax,pas)
//fmin : lower frequency or vector of input frequencies.
//[fmin, fmax, pas] : two real numbers (lower and upper frequency bounds). pas : logarithmic discretization step.
//fd matrix of output frequencies.
//dd matrix of magnitudes one row for each system (same size for fd matrix and dd matrix).
//pd matrix of phases one row for each system (same size for fd matrix and pd matrix).
[lhs,rhs]=argn(0);
eps = 1.e-10;
l10 = log(10);
if rhs <3 | rhs > 5 then
  error(msprintf(gettext("%s: Wrong number of input arguments : %d to %d expected.\n"),"dpf_cd",3,5))
  return
end
typc = typeof(gc);
if and(typc <> ["rational","state-space","rd","zpk"]) then
  error(msprintf(gettext("%s: Wrong type for input argument #%d: Linear dynamical system expected.\n"),"dpf_cd",1))
  return
end
select typc
  case "state-space" then
    gc = clean(ss2tf(gc));//see ss2tf function.
    domc = gc.dt;
  case "zpk" then
    gc = zpk2tf(gc);
    domc = gc.dt;
  case "rd" then //case "rd" : "iodelay toolbox".
    domc = gc.H.dt;
  case "rational" then  
    domc = gc.dt; //case "rational".
end
if domc <> "c" then
  error(msprintf(gettext("%s: Wrong type for argument #%d: In continuous time expected.\n") ,"dpf_cd",1))
  return
end
[lgc,cgc] = size(gc);
if cgc <> 1 then
  error(msprintf(gettext("%s: Wrong size for input argument #%d: SIMO linear dynamical system expected.\n"),"dpf_cd",1))
  return
end
//
typd = typeof(gd);
if and(typd <> ["rational","state-space","zpk"]) then
  error(msprintf(gettext("%s: Wrong type for input argument #%d: Linear dynamical system expected.\n"),"dpf_cd",1))
  return
end
select typd
  case "state-space" then
    gd = clean(ss2tf(gd));//see ss2tf function.
    domd = gd.dt;
  case "zpk" then
    gd = zpk2tf(gd);
    domd = gd.dt;
  case "rational" then  
    domd = gd.dt; //case "rational".
end
if domd == "d" then
  domd = 1;
end
if type(domd) <> 1 then
  error(msprintf(gettext("%s: Wrong type for argument #%d: In discrete time expected.\n") ,"dpf_cd",2))
  return
end
[lgd,cgd] = size(gd);
if cgd <> 1 then
  error(msprintf(gettext("%s: Wrong size for input argument #%d: SIMO linear dynamical system expected.\n"),"dpf_cd",2))
  return
end
nyq_f = 1/(2*domd);//Nyquist frequency.
//
select rhs
  case 3 then //[fd,dd,pd] = dpf_cd(gc,gd,fmin)
    if type(fmin) <> 1 then
      error(msprintf(gettext("%s: Wrong type for argument #%d: A scalar expected.\n") ,"dpf_cd",3))
    return
    end
    if fmin < 0 then
      error(msprintf(gettext("%s: Wrong value for argument #%d: A positive scalar expected.\n") ,"dpf_cd",3))
    return
    end
    [lfmin,cfmin] = size(fmin);
    if lfmin <> 1 then
      error(msprintf(gettext("%s: Wrong size for input argument #%d: A row vector expected.\n"),"dpf_cd",3))
      return
    end
    if fmin($) <= nyq_f then
      warning(msprintf(gettext("%s: The value of last frequency is smaller than Nyquist frequency.\n"), "dpf_cd"));
      nyq_f = fmin($);
    end
    f1 = fmin;
    f1(f1 < eps) = eps;
    fd1 = ones(lgc,1)*f1;
//One do a homothety on frequency vector for sampled systems gd only.    
//on fait homothétie sur le vecteur fréquence pour gd seulement.
    f2 = (((nyq_f-f1(1))/(f1($)-f1(1)))*(f1 - f1(1))+f1(1));
    fd2 = ones(lgd,1)*f2;
    fd = [f1;f2];
  case 4 then //[fd,dd,pd] = dpf_cd(gc,gd,fmin,fmax)
    if type(fmin) <> 1 then
      error(msprintf(gettext("%s: Wrong type for argument #%d: A scalar expected.\n") ,"dpf_cd",3))
    return
    end
    if fmin < 0 then
      error(msprintf(gettext("%s: Wrong value for argument #%d: A positive scalar expected.\n") ,"dpf_cd",3))
    return
    end
    if type(fmax) <> 1 then
      error(msprintf(gettext("%s: Wrong type for argument #%d: A scalar expected.\n") ,"dpf_cd",4))
    return
    end
    if fmax < 0 then
      error(msprintf(gettext("%s: Wrong value for argument #%d: A positive scalar expected.\n") ,"dpf_cd",4))
    return
    end
    if fmax < fmin then
      error(msprintf(gettext("%s: Wrong value for input arguments #%d and #%d: %s < %s expected.\n"),..
      "dpf_cd",3,4,"fmin","fmax"));
      return
    end
// S'il existe un système continu dans le vecteur gc tel que son modèle est constant
// alors on supprime ce système car calfrq(rd) donne un nombre de points très important
// sans justification : on discrétise donc avec le reste des systèmes mais on calculera le 
// gain et la phase de gc.
    gcc = gc;
    if typc == "rd" then
      i=find(degree(gcc.H.den)==0);
      gcc(i) = [];
      f1 = calfrqrd(gcc,fmin,fmax);
    else
      i=find(degree(gcc.den)==0);
      gcc(i) = [];
      f1 = calfrq(gcc,fmin,fmax);
    end
//fd matrix of frequencies.
    f1(f1 < eps) = eps;
    fd1 = ones(lgc,1)*f1;
    if fmax <= nyq_f then
      warning(msprintf(gettext("%s: The value of last frequency is smaller than Nyquist frequency.\n"), "dpf_cd"));
      nyq_f = f1($);
    end
    f2 = (((nyq_f-f1(1))/(f1($)-f1(1)))*(f1 - f1(1))+f1(1));
    fd2 = ones(lgd,1)*f2;
    fd = [fd1;fd2];
  case 5 then //[fd,dd,pd] = dpf_cd(gc,gd,fmin,fmax,pas).
    if type(fmin) <> 1 then
      error(msprintf(gettext("%s: Wrong type for argument #%d: A scalar expected.\n") ,"dpf_cd",3))
    return
    end
    if fmin < 0 then
      error(msprintf(gettext("%s: Wrong value for argument #%d: A positive scalar expected.\n") ,"dpf_cd",3))
    return
    end
    if type(fmax) <> 1 then
      error(msprintf(gettext("%s: Wrong type for argument #%d: A scalar expected.\n") ,"dpf_cd",4))
    return
    end
    if fmax < 0 then
      error(msprintf(gettext("%s: Wrong value for argument #%d: A positive scalar expected.\n") ,"dpf_cd",4))
    return
    end
    if fmax < fmin then
      error(msprintf(gettext("%s: Wrong value for input arguments #%d and #%d: %s < %s expected.\n"),..
      "dpf_cd",3,4,"fmin","fmax"));
      return
    end
    if type(pas) <> 1 then
      error(msprintf(gettext("%s: Wrong type for argument #%d: A scalar expected.\n") ,"dpf_cd",5))
    return
    end
    if pas < 0 then
      error(msprintf(gettext("%s: Wrong value for argument #%d: A positive scalar expected.\n") ,"dpf_cd",5))
      return
    end
    if pas > (fmax - fmin) then
      error(msprintf(gettext("%s: Wrong value for argument #%d: This scalar is too big.\n") ,"dpf_cd",5))
      return
    end
    f1 = [exp(l10*((log(fmin)/l10):pas:(log(fmax)/l10))),fmax];
    f1(f1 < eps) = eps;
    fd1 = ones(lgc,1)*f1;
    if fmax <= nyq_f then
      warning(msprintf(gettext("%s: The value of last frequency is smaller than Nyquist frequency.\n"), "dpf_cd"));
      nyq_f = f1($);
    end
    f2 = (((nyq_f-f1(1))/(f1($)-f1(1)))*(f1 - f1(1))+f1(1));
    fd2 = ones(lgd,1)*f2;
    fd = [fd1;fd2];
end    
//Now we compute the magnitude and phase with dbphifr.sci
[dd1,pd1] = dbphifr(gc,f1);
[dd2,pd2] = dbphifr(gd,f2);
dd = [dd1;dd2];
pd = [pd1;pd2];
//Presentation
if lhs ==1 then 
  fd = dd;
end
if lhs ==2 then 
  fd = dd; dd = pd;
end
endfunction














