// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//Cette fonction qui utilise la fonction "roots" a pour but de trouver les mul-
//tiplicité des racines par l'utilisation de la fonction "bezout" sur le 
//polynôme P(s) et le polynôme dérivé dP(s)/ds. Deux présentations sont 
//possibles : soit on répète la racine si elle est multiple, soit on donne le 
//vecteur des racines et le vecteur des multiplicités, 
//(une racine simple a une multiplicité de 1).
//
//This function uses programs "roots" and "bezout" and gives roots of a 
//polynomial and multiplicity of each one.
//s=%s;
//P=s**3*(s-2)**3*(s+1)*(s*s+s+1)**4*(s+2)**4*(s+6)*(s*s+1.5*s+1)**3
//Rac=mroots(P)
//polynôme de degré 26 : on retrouve bien les racines réelles et les complexes 
//avec la bonne multiplicité.
//
//P=s**3*(s-2)**3*(s+1)*(s*s+s+1)**4*(s+2)**4*(s+6)*(s*s+1.5*s+1)**3*(s*s+.9*s..
//+.5)**4*(s*s+2)**5, (degré 44)
//Rac1=mroots(P)
// ==================================================================== 
function [rm, r] = mroots(P, flag)
//We used flag for roots program, rm two columns matrix, r repeated roots (column vector). 
//  ******************Tests des arguments d'entrée.
//    
[lhs,rhs] = argn(0);
eps = 1.e-3;                                               
if type(P) <> 2 then //Il faut que P soit un polynôme.
    error(212,1)
    return                                      
end
if ~isreal(P) then //Le polynôme doit être à coefficients réels.
  error(msprintf(_("%s: Wrong type for input argument #%d: Real polynomial expected.\n"),"mroots",1))
    return
end
//
select rhs;
    case 1
      flag = "e";
    case 2
      if type(flag) <> 10 then
        flag = "e";
      else
        if part(flag,1) <> "f" then
          flag = "e";
        end
      end
end
//
//  ******************Fin des tests des arguments d'entrée.
//
//  ******************Cas particuliers de polynômes de degré 1 ou 2.
if degree(P) < 3 then
    R=roots(P,flag);
      if degree(P) == 1 then
        rm = [R,1];r = R;
      else
        if R(1,1) <> R(2,1) then
          rm = [R,[1;1]];r = R;
        else
          rm = [R(1,1),2];r = R;
        end
      end
    return
end
//
//  *****************Fin présentation des cas particuliers.
//
//  *****************Programme principal utilisant le programme bezout.sci.
//
coe = coeff(P);
//On recherche les racines nulles à partir des coefficients de P.
nz = min(find(coe <> 0)) - 1; 
coeP = coe(nz+1:$)/coe($);//on rend P unitaire.
P = poly(coeP,varn(P),"c");//Le polynôme de départ sans les racines nulles.
dP = derivat(P);
[g,U] = bezout(P,dP);
//
//  ******************Cas particulier : le polynôme n'a que des racines simples
//  ******************le degré de g est égal à 0.
if degree(g) == 0 then
    RR = roots(P,flag);
    if nz <> 0 then
      mk = [nz;ones(RR)];
      RR = [0;RR]; 
    else
      mk = ones(RR);
    end
//  ******************Fin du cas particulier.
//
else
//
//  *****************Cas général, le pgcd est un polynôme.
//
    Ux = - U(2,2);
    Vx = U(1,2);
    RR = roots(Ux,flag);//les racines sans leurs multiplicités.
    dUx = derivat(Ux);
    mk = horner(Vx/dUx,RR);
    mk = round(real(mk));//Les multiplicités correspondantes.
//
//On va faire un test sur la véracité de l'existence de racines multiples ceci 
//est du à la précision des résultats donnés par le programme "bezout".
//Voir explication avec le polynôme de Wilkinson qui pose problème : 
//P=poly([1:18],"s"), et les suivants (si flag =="f").
//Tests sur la somme et le produit des racines.
//
    Sum = sum(RR.*mk); Sumvrai = -coeP($-1);
    Pro = real(prod(RR.^mk)); Provrai = (-1)^degree(P)*coeP(1);
    if coeP($-1) == 0 then//Sumvrai peut être nulle.
      dSum = 0
    else
      dSum = (Sumvrai-Sum)/Sumvrai;
    end
    dPro = (Provrai-Pro)/Provrai;//Provrai est toujours différent de zéro.
    if and ([abs(dSum) < eps,abs(dPro) < eps]) == %F then
      warning("It seems that the polynomial has only singles roots : we use roots program.");
      RR = roots(P,flag); mk = ones(RR);
    end
//rajoutons les racines nulles à R si elles existent.
    if nz <> 0 then
      RR = [0;RR];
      mk = [nz;mk];
    end
end
//
//  ******************Classement des racines par valeur croissantes.
//  ******************Première présentation de la solution.
//Les racines réelles.
indxr = find(imag(RR) == 0);
if indxr == [] then 
    Rreal = [];mkr = [];
else
    Rreal = real(RR(indxr)) ; mkr = mk(indxr);
    [Rreal,j] =gsort(Rreal,"g","i")//Classement par valeurs croissantes.
    mkr = mkr(j);
end
//Les racines imaginaires.
indxi = find(imag(RR) <> 0);
if indxi == [] then 
  Rimag = [];mki = [];
else
  Rimag = RR(indxi) ; mki = mk(indxi);
  [Rimag,j] =gsort(Rimag,"g","i")//Classement par modules croissants.
  mki = mki(j);
end
rm = [[Rreal;Rimag],[mkr;mki]];//Roots with multiplicities (matrix (n,2)).
//  ******************Fin du classement par ordre croissant pour les racines 
//réelles puis imaginaires.
//  ******************Fin de la première présentation de la solution.
//
//  ******************Deuxième présentation de la solution .
//
    rac = rm(:,1); multi = real(rm(:,2));
    index = find(imag(rac) == 0);//les racines réelles.
    if index == [] then 
      R1real = [];multir = [];
    else
      racr = real(rac(index)); multir = multi(index);
      R1real = [];
      for i = 1:length(racr)
        racri = racr(i,1);multiri = multir(i,1);
        R1real = [R1real;racri*ones(multiri,1)];
      end
    end
    index = find(imag(rac) > 0);//Les imaginaires au dessus de l'axe réel.
    if index == [] then 
      R1imagp = [];multip = [];
    else
      R1imagp = rac(index);
      multip = multi(index);
    end
    R1imag = [];
    for i = 1:length(R1imagp)
      R1imagpi=R1imagp(i,1); multipi = multip(i,1);
      ai =[R1imagpi;conj(R1imagpi)];
      R1imagi = repmat(ai,multipi,1);// on reproduit le motif [raci;conj(raci)]
// multipi fois.
      R1imag = [R1imag;R1imagi];
    end
    r = [R1real;R1imag];//The vector of roots.
//******************Fin de la deuxième présentation de la solution.
//
endfunction





 

