// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016.
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ==================================================================== 
// Cette fonction calcule l'approximation de Padé (m/n) d'une fonction F(z)
// dont on donne le développement en série de N termes : attention N = n+m+1.
// m degré du numérateur, n celui du dénominateur.
// Cette fonction renvoie deux vecteurs lignes :
// coefficients respectifs du numérateur et dénominateur (pour pouvoir utiliser
// l'instruction scilab "poly([coe0,coe1,...],'s','c')").
// c est une ligne de coefficients du développement en série ou une fonction ou une chaîne de caractères.
// exemple : c = [1,-1,1/2,-1/6,1/24,-1/120,1/720,-1/5040,1/40320,-1/362880,1/3628800,-1/39916800,1/4.790D+08]
// pour exponentielle exp(-z),(N = m+n+1 = 13).
//
// This function computes Pade approximant of function F(z)
// Maclaurin serie is given by vector "c", m is the degree of numerator and n the degree of denominator
// of Pade approximant.
// This function return a vector [num, den] coefficients of numerator and denominator of Pade approximant.
// Example : c = [1,-1,1/2,-1/6,1/24,-1/120,1/720,-1/5040,1/40320,-1/362880,1/3628800,-1/39916800,1/4.790D+08]
// for exp(-z), (N = m+n+1 = 13), in this case.
// ====================================================================
function [num, den] = pade(m, n, c)
N = n+m+1;
if type(c) <> 1 then 
 c = maclau(c,N);
else
   if length(c) < N then
    error(msprintf(_("%s: Wrong size for input argument #%d: n+m+1 coefficients in the row vector expected.\n"), "pade",3));
    return
   end
   tolc =1.d-15*norm(c);  
   c(abs(c) < tolc) = 0;  
   k1 = max(find(c <> 0));
   if k1 >= N then
     c = c(1:N);
   else
     c = [c(1:k1),zeros(1,N-k1)];
   end   

end
 d = c(1:N);
[a,b] = size(d);
if a == 1 then
   d = d';
end 
if n == 0 then
   num = d; den = 1;  // Obvious!
end
//c'est ici que l'on choisit 'k' tel que d(k*s) donne abs(di)<=1 comparables. 
dd = d(find(d));
[mi,i]=max(abs(dd));  // We look for the maximal value of abs(dd).
if i == 1 then
   k = 1;
else
   k = exp(-log(mi)/(i-1)); // (i-1) is the position of maximum value of abs(d) in vector d.
   d = d.*(k**(0:1:m+n)');// We have change "s" in "k*s": all terms of new "d" are <=1.
end
A = zeros(m+n,m+n);  // Construction of global matrix A(n+m,n+m).
for j=1:n
 A(j:(n+m),j) = d(1:(n+m+1-j)); // Part of global matrix.
end
A1 = A(((m+1):(m+n)),(1:n));
rk = rank(A1);
if rk <> n then // Decrease n and m; depend of the rank of A1.
   warning(msprintf(_("%s: We decrease n and m  #%d.\n"),"pade",1));
   m = m - (n - rk);
   if m <= 0 then
      m = 0;
   end
   n = rk;
   A1 = A(((m+1):(m+n)),(1:n));
end
Q = -pinv(A1)*d((m+2):(n+m+1)); // We use function pinv (which use "svd" function).
//In fact for exp(-z), with program inv, matrix A1 is an ill-conditioned matrix : rcond=3.55D-11
//for n = 6 and m = 6 (N = 13, is the limit).
den = [1;Q]';
den = den.*((1/k)**(0:1:n));  // We change "k*s" in "s" for denominator of Pade approximant.
if m == 0 then
   num = d(1);
end
A2 = A((1:m),(1:n));
P = A2*Q+d(2:(m+1));
num = [d(1);P]';
num = num.*((1/k)**(0:1:m)); //We change "k*s" in "s" for numerator of Pade approximant.
endfunction
//
//
//
//Développement de Maclaurin d'une fonction par l'utilisation de la fft.
//Maclaurin developpement of function by fft.
function c = maclau(f,n)
typo = typeof(f);
if and(typo <> ["string","function"]) then
   error(msprintf(gettext("%s: Wrong type for input argument #%d: string or function expected.\n"),"maclau",1))
   return
end
l = 1024;
z = exp(2*%pi*%i*(0:l-1)'/l);
if typo == "string" then
   cz = evstr(f);
   ctayl = fft(cz)/l;
else
   ctayl =fft(f(z))/l;
end
tolct = 1.d-15*norm(ctayl);
ctayl(abs(ctayl) < tolct) = 0;
if norm(imag(ctayl),"inf") > tolct then 
   error(msprintf(gettext("%s: Wrong function for input argument #%d: this function must be analytic near unit disc .\n"),"maclau",1))
   return
end
ctayl = real(ctayl)';
k1 = max(find(ctayl <> 0));
if k1 >= n then
   c = ctayl(1:n);
else
   c = [ctayl(1:k1),zeros(1,n-k1)];
end
endfunction


